@@ -2,7 +2,7 @@
 # Copyright 2007-2012 Gentoo Foundation
 # Copyright 2007-2012 Mike Frysinger <vapier@gentoo.org>
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-projects/pax-utils/lddtree.sh,v 1.13 2012/11/04 07:26:24 vapier Exp $
+# $Header: /var/cvsroot/gentoo-projects/pax-utils/lddtree.sh,v 1.18 2013/01/02 17:37:27 vapier Exp $
 
 argv0=${0##*/}
 
@@ -19,12 +19,20 @@ usage() {
 	Options:
 	  -a          Show all duplicated dependencies
 	  -x          Run with debugging
-	  -h          Show this help output
 	  -R <root>   Use this ROOT filesystem tree
+      -l          Display output in a flat format
+	  -h          Show this help output
+	  -V          Show version information
 	EOF
 	exit ${1:-0}
 }
 
+version() {
+	local id='$Id: lddtree.sh,v 1.18 2013/01/02 17:37:27 vapier Exp $'
+	id=${id##*,v }
+	exec echo "lddtree-${id% * Exp*}"
+}
+
 error() {
 	echo "${argv0}: $*" 1>&2
 	ret=1
@@ -32,7 +40,10 @@ error() {
 }
 
 elf_specs() {
-	scanelf -BF '#F%a %M %D %I' "$1"
+	# With glibc, the NONE, SYSV, and LINUX OSABI's are compatible.
+	# NONE and SYSV are the same thing, so normalize LINUX to NONE. #442024
+	scanelf -BF '#F%a %M %D %I' "$1" | \
+		sed 's: LINUX$: NONE:'
 }
 
 lib_paths_fallback="${ROOT}lib* ${ROOT}usr/lib* ${ROOT}usr/local/lib*"
@@ -62,7 +73,8 @@ find_elf() {
 
 		if [[ ${c_last_needed_by} != ${needed_by} ]] ; then
 			c_last_needed_by=${needed_by}
-			c_last_needed_by_rpaths=$(scanelf -qF '#F%r' "${needed_by}" | sed 's|:| |g')
+			c_last_needed_by_rpaths=$(scanelf -qF '#F%r' "${needed_by}" | \
+				sed -e 's|:| |g' -e "s:[$]ORIGIN:${needed_by%/*}:")
 		fi
 		check_paths "${elf}" ${c_last_needed_by_rpaths} && return 0
 
@@ -72,7 +84,7 @@ find_elf() {
 			local p path=${LD_LIBRARY_PATH}
 			while : ; do
 				p=${path%%:*}
-				check_paths "${elf}" "${path:-${PWD}}" && return 0
+				check_paths "${elf}" "${p:-${PWD}}" && return 0
 				[[ ${path} == *:* ]] || break
 				path=${path#*:}
 			done
@@ -120,19 +132,27 @@ show_elf() {
 	resolved=${_find_elf}
 	elf=${elf##*/}
 
-	printf "%${indent}s%s => " "" "${elf}"
+	${LIST} || printf "%${indent}s%s => " "" "${elf}"
 	if [[ ,${parent_elfs}, == *,${elf},* ]] ; then
-		printf "!!! circular loop !!!\n" ""
+		${LIST} || printf "!!! circular loop !!!\n" ""
 		return
 	fi
 	parent_elfs="${parent_elfs},${elf}"
-	printf "${resolved:-not found}"
+	if ${LIST} ; then
+		echo "${resolved:-$1}"
+	else
+		printf "${resolved:-not found}"
+	fi
 	if [[ ${indent} -eq 0 ]] ; then
 		elf_specs=$(elf_specs "${resolved}")
 		interp=$(scanelf -qF '#F%i' "${resolved}")
 		[[ -n ${interp} ]] && interp="${ROOT}${interp#/}"
 
-		printf " (interpreter => ${interp:-none})"
+		if ${LIST} ; then
+			[[ -n ${interp} ]] && echo "${interp}"
+		else
+			printf " (interpreter => ${interp:-none})"
+		fi
 		if [[ -r ${interp} ]] ; then
 			# Extract the default lib paths out of the ldso.
 			lib_paths_ldso=$(
@@ -142,7 +162,7 @@ show_elf() {
 		fi
 		interp=${interp##*/}
 	fi
-	printf "\n"
+	${LIST} || printf "\n"
 
 	[[ -z ${resolved} ]] && return
 
@@ -168,13 +188,16 @@ if [[ $1 != "/../..source.lddtree" ]] ; then
 
 SHOW_ALL=false
 SET_X=false
+LIST=false
 
-while getopts haxR: OPT ; do
+while getopts haxVR:l OPT ; do
 	case ${OPT} in
 		a) SHOW_ALL=true;;
 		x) SET_X=true;;
 		h) usage;;
+		V) version;;
 		R) ROOT="${OPTARG%/}/";;
+		l) LIST=true;;
 		?) usage 1;;
 	esac
 done
