@@ -1,13 +1,13 @@
 /*
  * Copyright 2003-2012 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.250 2012/11/04 08:25:41 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.253 2012/11/30 23:25:07 vapier Exp $
  *
  * Copyright 2003-2012 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2004-2012 Mike Frysinger  - <vapier@gentoo.org>
  */
 
-static const char rcsid[] = "$Id: scanelf.c,v 1.250 2012/11/04 08:25:41 vapier Exp $";
+static const char rcsid[] = "$Id: scanelf.c,v 1.253 2012/11/30 23:25:07 vapier Exp $";
 const char argv0[] = "scanelf";
 
 #include "paxinc.h"
@@ -321,7 +321,6 @@ static char *scanelf_file_phdr(elfobj *elf, char *found_phdr, char *found_relro,
 	static char ret[12];
 	char *found;
 	unsigned long i, shown, multi_stack, multi_relro, multi_load;
-	int max_pt_load;
 
 	if (!show_phdr) return NULL;
 
@@ -329,7 +328,6 @@ static char *scanelf_file_phdr(elfobj *elf, char *found_phdr, char *found_relro,
 
 	shown = 0;
 	multi_stack = multi_relro = multi_load = 0;
-	max_pt_load = elf_max_pt_load(elf);
 
 #define NOTE_GNU_STACK ".note.GNU-stack"
 #define SHOW_PHDR(B) \
@@ -355,9 +353,6 @@ static char *scanelf_file_phdr(elfobj *elf, char *found_phdr, char *found_relro,
 				offset = 4; \
 				check_flags = PF_X; \
 			} else if (EGET(phdr[i].p_type) == PT_LOAD) { \
-				if (EGET(ehdr->e_type) == ET_DYN || EGET(ehdr->e_type) == ET_EXEC) \
-					if (multi_load++ > max_pt_load) \
-						warnf("%s: more than %i PT_LOAD's !?", elf->filename, max_pt_load); \
 				if (file_matches_list(elf->filename, qa_wx_load)) \
 					continue; \
 				found = found_load; \
@@ -770,6 +765,7 @@ static void scanelf_file_rpath(elfobj *elf, char *found_rpath, char **ret, size_
 		xstrcat(ret, "  -  ", ret_len);
 }
 
+/* Defines can be seen in glibc's sysdeps/generic/ldconfig.h */
 #define LDSO_CACHE_MAGIC "ld.so-"
 #define LDSO_CACHE_MAGIC_LEN (sizeof LDSO_CACHE_MAGIC -1)
 #define LDSO_CACHE_VER "1.7.0"
@@ -788,6 +784,9 @@ static void scanelf_file_rpath(elfobj *elf, char *found_rpath, char **ret, size_
 #define FLAG_POWERPC_LIB64  0x0500
 #define FLAG_MIPS64_LIBN32  0x0600
 #define FLAG_MIPS64_LIBN64  0x0700
+#define FLAG_X8664_LIBX32   0x0800
+#define FLAG_ARM_LIBHF      0x0900
+#define FLAG_AARCH64_LIB64  0x0a00
 
 #if defined(__GLIBC__) || defined(__UCLIBC__)
 
@@ -1022,7 +1021,24 @@ static char *scanelf_file_interp(elfobj *elf, char *found_interp)
 		}
 		SHOW_INTERP(32)
 		SHOW_INTERP(64)
+	} else {
+		/* Walk all the program headers to find the PT_INTERP */
+#define SHOW_PT_INTERP(B) \
+		if (elf->elf_class == ELFCLASS ## B) { \
+		unsigned long i; \
+		Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
+		Elf ## B ## _Phdr *phdr = PHDR ## B (elf->phdr); \
+		for (i = 0; i < EGET(ehdr->e_phnum); i++) { \
+			if (EGET(phdr[i].p_type) != PT_INTERP) \
+				continue; \
+			*found_interp = 1; \
+			return (be_wewy_wewy_quiet ? NULL : elf->data + EGET(phdr[i].p_offset)); \
+		} \
+		}
+		SHOW_PT_INTERP(32)
+		SHOW_PT_INTERP(64)
 	}
+
 	return NULL;
 }
 static char *scanelf_file_bind(elfobj *elf, char *found_bind)
